@@ -8,7 +8,7 @@ use JSON::Syck qw(Dump);
 use Encode qw(decode encode);
 use Smart::Comments;
 
-print "姓名|工号|部门|邮箱|分机号|手机|雅虎通|职位|性别|工作地|\n";
+print "姓名|工号|部门|邮箱|分机号|手机|雅虎通|职位|性别|工作地|序号|\n";
 while (<>) {
     if (s/^INSERT INTO `rolldata` VALUES //) {
         #$_ = dtf8', $_);
@@ -28,7 +28,18 @@ while (<>) {
             my $phone = $vals[15];
             my $cell= $vals[16];
             $pos =~ s/高级|资深|首席//g;
-            print "$name|$emp_id|$depart|$email|$phone|$cell|$yid|$pos|$sex|$place|\n";
+            $pos = '' if $pos eq '总监';
+            my $order = 0;
+            if ($depart =~ /雅虎/) {
+                $order++;
+                if ($depart =~ /搜索/) {
+                    $order++;
+                }
+            }
+            if ($name =~ /张皛珏/) {
+                $order = 2;
+            }
+            print "$name|$emp_id|$depart|$email|$phone|$cell|$yid|$pos|$sex|$place|$order\n";
         }
     }
 }
