@@ -147,7 +147,7 @@ _EOC_
     ],
     [ '0.005' => '' ],
     [ '0.006' => '' ],  # v0.4.0
-    [ '0.007' => '' ],  # v0.5.0
+    [ '0.007' => '' ],  # v0.5.0 & v0.5.1
 );
 
 our @LocalVersionDelta = (
