@@ -16,6 +16,7 @@ use File::Spec;
 
 our $InitFatal;
 our $StatsLog;
+our $Context;
 
 # XXX Excpetion not caputred...when database 'test' not created.
 
@@ -25,15 +26,28 @@ if ($url_prefix) {
 }
 
 sub init {
-    my ($class, $context) = @_;
-    #warn "init: $backend\n";
+    my ($class, $opts) = @_;
+    
+    my $context = $opts->{context};
+    if (defined $context) {
+        $Context = $context;
+    } else {
+        $context = $Context;
+    }
+    
+    undef $InitFatal;
+
     eval {
-        OpenResty::Config->init;
+        OpenResty::Config->init($opts);
         my $backend = $OpenResty::Config{'backend.type'};
         $OpenResty::Cache = OpenResty::Cache->new;
         OpenResty->connect($backend);
     };
-    if ($@) { $InitFatal = $@; return; }
+    if ($@) { 
+        # warn $@; 
+        $InitFatal = $@; 
+        return; 
+    }
     #warn "InitFatal: $InitFatal\n";
 
     if (!$context || ($context ne 'upgrade' && $context !~ /user/)) {
@@ -106,6 +120,13 @@ sub init {
 sub process_request {
     my ($class, $cgi, $call_level, $parent_account) = @_;
 
+    if ($InitFatal) {
+        # warn "Init error: $InitFatal";
+        warn "Found init fatal error. Now we re-init the dispatcher...\n";
+        
+        $class->init({'context' => $Context});
+    }
+
     $call_level ||= 0;
 
     if ($call_level > $ACTION_REC_DEPTH_LIMIT) {
@@ -158,15 +179,21 @@ sub process_request {
 
     map { s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg; } @bits;
     ## @bits
-
+    
     my $fst = shift @bits;
     if ($fst ne '=') {
         return $openresty->fatal("URLs must be led by '=': $url");
     }
-
+    
     my $key = $bits[0];
     if (!defined $key) { $key = $bits[0] = 'version'; }
 
+    if (scalar(@bits) == 4 && $bits[2] =~ m{^_\w+} ) {
+        # warn $openresty->{_builtin_params};
+        $openresty->{_builtin_params}->{$bits[2]} = $bits[3]; 
+        $bits[2] = '~';
+        $bits[3] = '~';
+    }
     my $http_meth = $openresty->{'_http_method'};
     if (!$http_meth) {
         return $openresty->fatal("HTTP method not detected.");
